`define ADDR_WIDTH_M1 2
`define DATA_WIDTH_M1 15

`timescale 10ps/1ps
module FIFO_tb1();                           
    reg reset  = 0;
    reg clk_wr = 0;
    reg clk_rd = 0;
    reg wr_en  = 0;
    reg rd_en  = 0;

    reg [`ADDR_WIDTH_M1: 0] count;
    reg [`DATA_WIDTH_M1: 0] data_in = 0;

    wire wr_request;
    wire [`DATA_WIDTH_M1: 0] data_out;
    wire rd_request;

    // Submodule
    FIFO DUT (
    .reset      (reset),           // synchronous to read clock  --------------------------
    .clk_wr     (clk_wr),          // clock coming from write side of FIFO -- write signals
    .wr_en      (wr_en),           // write enable
    .data_in    (data_in),         // data to be written 
    .wr_request (wr_request),      // low= Full, else NOT FULL
    .clk_rd     (clk_rd),          // clock coming from read side of FIFO  -- read signals 
    .rd_en      (rd_en),           // read enable
    .data_out   (data_out),        // data to be read  
    .rd_request (rd_request)       // FIFO is not empty (combinational in from 1st stage of FIFO)
    );

    // Initialization for data capture
    initial
    begin
        //$recordfile("tb");
        //$recordvars(tb);

        $dumpfile("FIFO.vcd"); 
        $dumpvars(0, FIFO_tb1); 

        #0
        reset = 0;
        clk_wr = 0;
        clk_rd = 0;
        wr_en = 0;
        rd_en = 0;
        count = 0;
        data_in = 0;
         
        #201
        reset = 0;
        wr_en = 1;
        rd_en = 1;

        #40000

        #100
        $finish;
    end
  
    always @ (negedge clk_wr)
    begin
        if ((wr_request) && (wr_en))
        begin
            count = count + 1;
        end
        data_in = count;
    end

    always
        #300 clk_wr = !clk_wr;

    always
        #100 clk_rd = !clk_rd;

endmodule

